@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -37,8 +37,9 @@
 /* ======================================================================== */
 /* configure */
 
-//#define KNH_USING_RCGC       1
-#define KNH_USING_UNBOXFIELD 1
+#define KNH_USING_RCGC         1
+#define KNH_USING_FASTMALLOC   1
+#define KNH_USING_UNBOXFIELD   1
 
 /* ======================================================================== */
 /* Manifesto */
@@ -48,28 +49,36 @@
 #define KONOHA_VERSION   PACKAGE_VERSION
 #else
 #define KONOHA_NAME     "konoha"
-#define KONOHA_MAJOR_VERSION "0.5"
-#define KONOHA_VERSION  "0.5.0"
+#define KONOHA_MAJOR_VERSION  0
+#define KONOHA_MINER_VERSION  5
+#define KONOHA_PATCH_VERSION  0
+#define KONOHA_VERSION  "0.5"
 #endif
 
-#define KONOHA_XCODE    "Toguro"
+#define LIBKONOHA_VERSION "0.5"
+
+#define KONOHA_XCODE    "TOGURO"
 #define KONOHA_URL		"http://konoha.sourceforge.jp/"
 #define KONOHA_URLBASE  "kttp://konoha.sourceforge.jp/wiki/"
 
-#define KONOHA_BUILDID_TOOOLD  545
-#define KNH_CHECK_VERSION(id)  ((id) <= KONOHA_BUILDID_TOOOLD)
-
 /* ======================================================================== */
 /* COMMON */
 
-#define KONOHA_TSTRUCT_SIZE                64
-#define KONOHA_TCLASS_SIZE                 256
-#define KONOHA_TEXP_SIZE                   128
-
-/* object specification */
-#define KONOHA_PAGESIZE                    4096
-#define KONOHA_FIELDSIZE                   64
+#if defined(__LP64__)
+#define KONOHA_TSTRUCT_SIZE                 512
+#define KONOHA_TCLASS_SIZE                 4096
+#define KONOHA_TEXP_SIZE                    128
+#define KONOHA_PAGESIZE                    8192
+#define KONOHA_FIELDSIZE                     64
 #define KONOHA_STACKSIZE                   4096
+#else
+#define KONOHA_TSTRUCT_SIZE                  64
+#define KONOHA_TCLASS_SIZE                  256
+#define KONOHA_TEXP_SIZE                    128
+#define KONOHA_PAGESIZE                    4096
+#define KONOHA_FIELDSIZE                     64
+#define KONOHA_STACKSIZE                   1024
+#endif
 
 #define KONOHA_SMALLPAGESIZE               256
 
@@ -78,8 +87,6 @@
 #define KONOHA_DEFAULT_ARRAY_SIZE           16
 #define KONOHA_STRING_BUFSIZ               128
 #define CLASSNAME_BUFSIZ                   128
-#define FILENAME_BUFSIZ                    512
-#define ERRMSG_BUFSIZ                      512
 
 /* object specification */
 
@@ -88,34 +95,44 @@
 
 /* ======================================================================== */
 
-#ifdef KONOHA_OS__LINUX
+#ifdef KONOHA_ON_LINUX
 #define KONOHA_OS "linux"
 #endif
 
-#ifdef KONOHA_OS__CYGWIN
+#ifdef KONOHA_ON_CYGWIN
 #define KONOHA_OS "cygwin"
 #endif
 
-#ifdef KONOHA_OS__MINGW
+#ifdef KONOHA_ON_MINGW
 #define KONOHA_OS "mingw"
 #endif
 
-#ifdef KONOHA_OS__MACOSX
+#ifdef KONOHA_ON_MACOSX
 #define KONOHA_OS "macosx"
 #endif
 
-#ifdef KONOHA_OS__TB
+#ifdef KONOHA_ON_TB
 #define KONOHA_OS "teaboard"
 #endif
 
+#define KNH_USING_STDC   1
+
+#ifdef KONOHA_ON_LKM
+	#undef KNH_USING_STDC
+	#define KONOHA_OS "LKM"
+#endif
+
+
 /* ======================================================================== */
 /* [CC] */
 /* cpp -dM /dev/null */
 
 #ifdef __GNUC__
 	#define KONOHA_CC_VERSION "GCC " __VERSION__
-	#ifdef __i386__
-		#define KNH_CC_FASTCALL   __attribute__((fastcall))
+	#if defined(__i386__) || (defined(__x86_64__) && !defined(__amd64__))
+		#undef fastcall
+		#define fastcall __attribute__((fastcall))
+		#define KNH_CC_FASTCALL   fastcall
 		#define likely(x)       __builtin_expect(!!(x), 1)
 		#define unlikely(x)     __builtin_expect(!!(x), 0)
 	#else
@@ -125,26 +142,25 @@
 	#ifndef KONOHA_OS
 		#ifdef __linux__
 			#define KONOHA_OS  "linux"
-			#define KONOHA_OS__LINUX       1
+			#define KONOHA_ON_LINUX       1
 		#endif
 		#ifdef __APPLE__
 			#define KONOHA_OS  "macosx"
-			#define KONOHA_OS__MACOSX      1
+			#define KONOHA_ON_MACOSX      1
 		#endif
 		#ifdef __WIN32__
 			#define KONOHA_OS  "mingw"
-			#define KONOHA_OS__MINGW          1
+			#define KONOHA_ON_MINGW          1
 		#endif
 		#ifdef __CYGWIN__
 			#define KONOHA_OS  "cygwin"
-			#define KONOHA_OS__CYGWIN     1
+			#define KONOHA_ON_CYGWIN     1
 		#endif
 		#ifdef TBMX1
 			#define KONOHA_OS  "teaboard"
-			#define KONOHA_OS__TB       1
+			#define KONOHA_ON_TB       1
 		#endif
 	#endif/*KONOHA_OS*/
-
 #endif/*__GNUC__*/
 
 #ifdef _MSC_VER
@@ -157,14 +173,14 @@
 #endif
 	#ifndef KONOHA_OS
 		#define KONOHA_OS  "windows"
-		#define KONOHA_OS__WINDOWS
+		#define KONOHA_ON_WINDOWS
 	#endif
 #define _CRT_SECURE_NO_WARNINGS
 #endif  /* _MSC_VER */
 
 #ifndef KONOHA_OS
 	#define KONOHA_OS  "unknown"
-	#define KONOHA_OS__UNKNOWN
+	#define KONOHA_ON_UNKNOWN
 #endif
 
 #ifndef KONOHA_CC_VERSION
@@ -172,8 +188,8 @@
 #endif
 
 #ifndef likely
-#define likely(x)	   x
-#define unlikely(x)	   x
+#define likely(x)	   (x)
+#define unlikely(x)	   (x)
 #endif
 
 #ifndef KONOHA_CPU
@@ -186,20 +202,13 @@
 #define KONOHA_PLATFORM KONOHA_OS "_32"
 #endif
 
+#ifdef __SSE2__
+#define KNH_USING_SSE2
+#endif
+
 /* ======================================================================== */
 /* [OPTIONS_DEPS] */
 
-#define KNH_USING_RCGC 1
-
-#ifdef KNH_USING_DRCGC
-#define KNH_USING_RCGC 1
-#endif
-
-#ifdef KONOHA_CC_WITH_ECLIPSE
-	#define KONOHA_MONOLITHIC  1
-	#define KNH_USING_MATH    32
-#endif
-
 /* ======================================================================== */
 
 #define KNH_SECUREMODE 1
@@ -208,29 +217,11 @@
 
 /* ======================================================================== */
 
-#ifndef L1_CACHE_BYTES
+#if !defined(L1_CACHE_BYTES) && !defined(KONOHA_ON_LKM)
 #define L1_CACHE_BYTES   32
 #endif
 
-#ifdef KNH_FASTMODE
-#define KNH_USING_FASTMALLOC 1
-#define KNH_USING_OBJECT32   (sizeof(void*)*8)
-//#define KNH_USING_WORDKLRC   1
-#endif
-
-
-/* ======================================================================== */
-/* Language Specification */
-
 /* ======================================================================== */
 /* ======================================================================== */
 
-/* Runtime */
-#define KONOHA_OBJECT_COPY                 1
-#define KONOHA_RUNTIME_SINGLE              1
-
-/* ======================================================================== */
-/* ======================================================================== */
-
-
 #endif /*KONOHA_CONFIG_H_*/
