@@ -38,65 +38,82 @@ extern "C" {
 #ifdef KNH_CC_METHODAPI
 
 /* ------------------------------------------------------------------------ */
-/* @method Connection! Connection.new(String! urn) */
+//## method This! Connection.new(String! urn);
 
-static METHOD knh__Connection_new(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD Connection_new(Ctx *ctx, knh_sfp_t *sfp)
 {
-	Connection *o = (Connection*)sfp[0].o;
-	knh_Connection_open(ctx, o, (String*)sfp[1].s);
+	knh_Connection_t *o = (knh_Connection_t*)sfp[0].o;
+	knh_Connection_open(ctx, o, (knh_String_t*)sfp[1].s);
 	KNH_RETURN(ctx, sfp, sfp[0].o);
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method ResultSet! Connection.query(String! query) */
+//## method ResultSet! Connection.query(String! query);
 
-static METHOD knh__Connection_query(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD Connection_query(Ctx *ctx, knh_sfp_t *sfp)
 {
-	Connection *o = (Connection*)sfp[0].o;
-	String *query = (String*)sfp[1].o;
-	ResultSet *rs = new_ResultSet(ctx);
+	knh_Connection_t *c = (knh_Connection_t*)sfp[0].o;
+	knh_String_t *query = (knh_String_t*)sfp[1].o;
+	knh_ResultSet_t *rs = new_ResultSet(ctx);
 	KNH_RCSETv(ctx, sfp[2].o, rs);
-	knh_dbcur_t *dbcur = DP(o)->df->dbquery(ctx, DP(o)->conn, knh_String_tobytes(query), rs);
+	knh_dbcur_t *dbcur = (c)->df->dbquery(ctx, (c)->conn, __tobytes(query), rs);
 	if(dbcur != NULL) {
 		DP(rs)->dbcur = dbcur;
-		DP(rs)->dbcur_free = DP(o)->df->dbcurfree;
+		DP(rs)->dbcur_free = (c)->df->dbcurfree;
 	}
 	else {
 		DP(rs)->dbcur = NULL;
 		DP(rs)->dbcur_free = knh_dbcurfree__NOP;
 	}
-	KNH_SETv(ctx, DP(rs)->conn, o);
+	KNH_SETv(ctx, DP(rs)->conn, c);
 	KNH_RETURN(ctx, sfp, rs);
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void Connection.exec(String! query) */
+//## method void Connection.exec(String! query);
 
-static METHOD knh__Connection_exec(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD Connection_exec(Ctx *ctx, knh_sfp_t *sfp)
 {
-	Connection *o = (Connection*)sfp[0].o;
-	String *query = (String*)sfp[1].o;
-	DP(o)->df->dbquery(ctx, DP(o)->conn, knh_String_tobytes(query), NULL);
+	knh_Connection_t *c = (knh_Connection_t*)sfp[0].o;
+	knh_String_t *query = (knh_String_t*)sfp[1].o;
+	(c)->df->dbquery(ctx, (c)->conn, __tobytes(query), NULL);
 	KNH_RETURN_void(ctx, sfp);
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method Int! ResultSet.getSize(Any! n) */
+//## method void Connection.close();
 
-static METHOD knh__ResultSet_getSize(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD Connection_close(Ctx *ctx, knh_sfp_t *sfp)
 {
-	ResultSet *o = (ResultSet*)sfp[0].o;
+	knh_Connection_close(ctx, (knh_Connection_t*)sfp[0].o);
+	KNH_RETURN_void(ctx, sfp);
+}
+
+/* ------------------------------------------------------------------------ */
+//## method Int! ResultSet.getSize();
+
+static METHOD ResultSet_getSize(Ctx *ctx, knh_sfp_t *sfp)
+{
+	knh_ResultSet_t *o = (knh_ResultSet_t*)sfp[0].o;
 	KNH_RETURN_Int(ctx, sfp, DP(o)->column_size);
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method String! ResultSet.getName(Int! n) */
+//## method Boolean! ResultSet.next();
 
-static METHOD knh__ResultSet_getName(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD ResultSet_next(Ctx *ctx, knh_sfp_t *sfp)
 {
-	ResultSet *o = (ResultSet*)sfp[0].o;
+	KNH_RETURN_Boolean(ctx, sfp, knh_ResultSet_next(ctx, (knh_ResultSet_t*)sfp[0].o))
+}
+
+/* ------------------------------------------------------------------------ */
+//## method String! ResultSet.getName(Int! n);
+
+static METHOD ResultSet_getName(Ctx *ctx, knh_sfp_t *sfp)
+{
+	knh_ResultSet_t *o = (knh_ResultSet_t*)sfp[0].o;
 	size_t n = p_size(sfp[1]);
-	String *v = TS_EMPTY;
+	knh_String_t *v = TS_EMPTY;
 	if(n < DP(o)->column_size) {
 		v = knh_ResultSet_getName(ctx, o, n);
 	}
@@ -110,36 +127,38 @@ static METHOD knh__ResultSet_getName(Ctx *ctx, knh_sfp_t *sfp)
 
 int knh_ResultSet_indexof_(Ctx *ctx, knh_sfp_t *sfp)
 {
-	ResultSet *o = (ResultSet*)sfp[0].o;
+	knh_ResultSet_t *o = (knh_ResultSet_t*)sfp[0].o;
 	if(IS_bInt(sfp[1].o)) {
 		size_t n = p_size(sfp[1]);
 		if(!(n < DP(o)->column_size)) {
 			KNH_THROW_OUTOFINDEX(ctx, p_integer(sfp[1]), DP(o)->column_size);
-			KNH_STUPID(ctx, o, STUPID_OUTOFINDEX);
 			return -1;
 		}
 		return n;
 	}
 	else if(IS_bString(sfp[1].o)) {
-		int loc = knh_ResultSet_findColumn(ctx, o, knh_String_tobytes(sfp[1].s));
+		int loc = knh_ResultSet_findColumn(ctx, o, __tobytes(sfp[1].s));
 		if(loc == -1) {
-			KNH_STUPID(ctx, o, STUPID_NOTFOUND);
+			TODO();
+			//KNH_STUPID(ctx, o, STUPID_NOTFOUND);
 		}
 		return loc;
 	}
-	KNH_STUPID(ctx, o, STUPID_TYPEERROR);
+	TODO();
+	//KNH_STUPID(ctx, o, STUPID_NOTFOUND);
 	return -1;
 }
-/* ------------------------------------------------------------------------ */
-/* @method Int ResultSet.getInt(Any! n) */
 
-static METHOD knh__ResultSet_getInt(Ctx *ctx, knh_sfp_t *sfp)
+/* ------------------------------------------------------------------------ */
+//## method Int ResultSet.getInt(Any n);
+
+static METHOD ResultSet_getInt(Ctx *ctx, knh_sfp_t *sfp)
 {
 	int n = knh_ResultSet_indexof_(ctx, sfp);
 	knh_int_t res = 0;
 	if(n >= 0) {
-		ResultSet *o = (ResultSet*)sfp[0].o;
-		char *p = knh_Bytes_tochar(DP(o)->databuf) + DP(o)->column[n].start;
+		knh_ResultSet_t *o = (knh_ResultSet_t*)sfp[0].o;
+		char *p = knh_Bytes__tochar(DP(o)->databuf) + DP(o)->column[n].start;
 		switch(DP(o)->column[n].ctype) {
 		case knh_ResultSet_CTYPE__integer :
 			res = *((knh_int_t*)p);
@@ -157,15 +176,15 @@ static METHOD knh__ResultSet_getInt(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method Float ResultSet.getFloat(Any! n) */
+//## method Float ResultSet.getFloat(Any n);
 
-static METHOD knh__ResultSet_getFloat(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD ResultSet_getFloat(Ctx *ctx, knh_sfp_t *sfp)
 {
 	int n = knh_ResultSet_indexof_(ctx, sfp);
 	knh_float_t res = 0;
 	if(n >= 0) {
-		ResultSet *o = (ResultSet*)sfp[0].o;
-		char *p = knh_Bytes_tochar(DP(o)->databuf) + DP(o)->column[n].start;
+		knh_ResultSet_t *o = (knh_ResultSet_t*)sfp[0].o;
+		char *p = knh_Bytes__tochar(DP(o)->databuf) + DP(o)->column[n].start;
 		switch(DP(o)->column[n].ctype) {
 		case knh_ResultSet_CTYPE__integer :
 			res = (knh_float_t)(*((knh_int_t*)p));
@@ -183,40 +202,40 @@ static METHOD knh__ResultSet_getFloat(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method String ResultSet.getString(Any! n) */
+//## method String ResultSet.getString(Any n);
 
-static METHOD knh__ResultSet_getString(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD ResultSet_getString(Ctx *ctx, knh_sfp_t *sfp)
 {
 	int n = knh_ResultSet_indexof_(ctx, sfp);
 	Object *v = KNH_NULL;
 	if(n >= 0) {
-		v = UP(knh_ResultSet_getString(ctx, (ResultSet*)sfp[0].o, n));
+		v = UP(knh_ResultSet_getString(ctx, (knh_ResultSet_t*)sfp[0].o, n));
 	}
 	KNH_RETURN(ctx, sfp, v);
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method Any ResultSet.get(Any! n) */
+//## method Any ResultSet.get(Any n);
 
-static METHOD knh__ResultSet_get(Ctx *ctx, knh_sfp_t *sfp)
+static METHOD ResultSet_get(Ctx *ctx, knh_sfp_t *sfp)
 {
 	int n = knh_ResultSet_indexof_(ctx, sfp);
 	Object *v = KNH_NULL;
 	if(n >= 0) {
-		ResultSet *o = (ResultSet*)sfp[0].o;
-		char *p = knh_Bytes_tochar(DP(o)->databuf) + DP(o)->column[n].start;
+		knh_ResultSet_t *o = (knh_ResultSet_t*)sfp[0].o;
+		char *p = knh_Bytes__tochar(DP(o)->databuf) + DP(o)->column[n].start;
 		switch(DP(o)->column[n].ctype) {
 		case knh_ResultSet_CTYPE__integer :
 			KNH_RETURN_NNInt(ctx, sfp, (*((knh_int_t*)p)));
 		case knh_ResultSet_CTYPE__float :
 			KNH_RETURN_NNFloat(ctx, sfp, (*((knh_float_t*)p)));
 		case knh_ResultSet_CTYPE__text :
-			v = UP(new_String(ctx, B2(knh_Bytes_tochar(DP(o)->databuf) + DP(o)->column[n].start, DP(o)->column[n].len), NULL));
+			v = UP(new_String(ctx, B2(knh_Bytes__tochar(DP(o)->databuf) + DP(o)->column[n].start, DP(o)->column[n].len), NULL));
 			break;
 		case knh_ResultSet_CTYPE__bytes :
 			{
-				Bytes *ba = new_Bytes(ctx, DP(o)->column[n].len);
-				knh_Bytes_write(ctx, ba, B2(knh_Bytes_tochar(DP(o)->databuf) + DP(o)->column[n].start, DP(o)->column[n].len));
+				knh_Bytes_t *ba = new_Bytes(ctx, DP(o)->column[n].len);
+				knh_Bytes_write(ctx, ba, B2(knh_Bytes__tochar(DP(o)->databuf) + DP(o)->column[n].start, DP(o)->column[n].len));
 				v = UP(ba);
 			}
 			break;
@@ -228,9 +247,9 @@ static METHOD knh__ResultSet_get(Ctx *ctx, knh_sfp_t *sfp)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void ResultSet.%dump(OutputStream w, String m) */
+//## method void ResultSet.%dump(OutputStream w, String? m);
 
-static void knh_ResultSet__dump(Ctx *ctx, ResultSet *o, OutputStream *w, String *m)
+static void knh_ResultSet__dump(Ctx *ctx, knh_ResultSet_t *o, knh_OutputStream_t *w, knh_String_t *m)
 {
 	knh_putc(ctx, w, '{');
 	size_t n;
@@ -238,9 +257,9 @@ static void knh_ResultSet__dump(Ctx *ctx, ResultSet *o, OutputStream *w, String
 		if(n > 0) {
 			knh_write_delim(ctx,w);
 		}
-		knh_write(ctx, w, knh_String_tobytes(DP(o)->column[n].name));
+		knh_write(ctx, w, __tobytes(DP(o)->column[n].name));
 		knh_printf(ctx, w, "(%d): ", n);
-		char *p = knh_Bytes_tochar(DP(o)->databuf) + DP(o)->column[n].start;
+		char *p = knh_Bytes__tochar(DP(o)->databuf) + DP(o)->column[n].start;
 		switch(DP(o)->column[n].ctype) {
 			case knh_ResultSet_CTYPE__null :
 				knh_write(ctx, w, STEXT("null"));
@@ -262,6 +281,15 @@ static void knh_ResultSet__dump(Ctx *ctx, ResultSet *o, OutputStream *w, String
 	knh_putc(ctx, w, '}');
 }
 
+/* ------------------------------------------------------------------------ */
+//## method void ResultSet.close();
+
+static METHOD ResultSet_close(Ctx *ctx, knh_sfp_t *sfp)
+{
+	knh_ResultSet_close(ctx, (knh_ResultSet_t*)sfp[0].o);
+	KNH_RETURN_void(ctx, sfp);
+}
+
 /* ------------------------------------------------------------------------ */
 
 #endif
