@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -35,50 +35,6 @@
 extern "C" {
 #endif
 
-/* ======================================================================== */
-/* [macros] */
-
-#define _knh_Object_cid(o)           (o)->h.cid
-#define _knh_Object_bcid(o)          (o)->h.bcid
-
-/* ======================================================================== */
-/* [macros] */
-
-#define _IS_NULL(o)          (((Object*)o)->h.cid == CLASS_Nue)
-#define _IS_NOTNULL(o)       (((Object*)o)->h.cid != CLASS_Nue)
-
-#define _IS_TRUE(o)         ((o)->h.bcid == CLASS_Boolean && ((Int*)o)->n.bvalue)
-#define _IS_FALSE(o)        ((o)->h.bcid == CLASS_Boolean && (((Int*)o)->n.bvalue == 0))
-#define _new_Boolean(ctx, c)    (c) ? KNH_TRUE : KNH_FALSE
-
-#define _BOOL_ISTRUE(o)         (o == KNH_TRUE)
-#define _BOOL_ISFALSE(o)        (o == KNH_FALSE)
-
-///* ------------------------------------------------------------------------ */
-//
-//knh_Object_t *new_Boolean(Ctx *ctx, int c)
-//{
-//	return (c) ? KNH_TRUE : KNH_FALSE;
-//}
-
-/* ======================================================================== */
-/* [Null] */
-
-/* ------------------------------------------------------------------------ */
-
-Object *new_Nue(Ctx *ctx, String *msg)
-{
-	KNH_ASSERT(IS_String(msg));
-	knh_Nue_t *o = (knh_Nue_t*)new_hObject(ctx, FLAG_Nue, CLASS_Nue, CLASS_Nue);
-	KNH_INITv(o->orign, msg);
-	o->str = (knh_uchar_t*)knh_String_tochar(msg);
-	o->size = knh_String_strlen(msg);
-	return (Object*)o;
-}
-
-#define _new_Nue__b(ctx, txt)   new_Nue(ctx, new_String(ctx, txt, NULL));
-#define _new_Nue__s(ctx, txt)   new_Nue(ctx, new_String(ctx, B(txt), NULL));
-
 /* ======================================================================== */
 /* [Glue] */
 
@@ -93,7 +49,6 @@ void knh_fgfree__default(Ctx *ctx, knh_Glue_t *g)
 
 KNHAPI(void) knh_Glue_init(Ctx *ctx, knh_Glue_t *g, void *ptr, knh_fgfree gfree)
 {
-	DBG_P("bcid=%s, cid=%s", CLASSN(g->h.bcid), CLASSN(g->h.cid));
 	KNH_ASSERT(IS_bAny(g));
 	g->ptr = ptr;
 	if(gfree == NULL) gfree = knh_fgfree__default;
@@ -104,44 +59,30 @@ KNHAPI(void) knh_Glue_init(Ctx *ctx, knh_Glue_t *g, void *ptr, knh_fgfree gfree)
 
 KNHAPI(Object*) new_Glue(Ctx *ctx, char *lname, void *ptr, knh_fgfree gfree)
 {
-	knh_class_t cid = konoha_getcid(ctx, B(lname));
+	knh_class_t cid = knh_getcid(ctx, B(lname));
 	if(cid == CLASS_unknown) {
-		KNH_THROWf(ctx, "Unknown!!: %s", lname);
+		fprintf(stderr, "Unknown glue class: %s", lname);
+		KNH_BUGSTOP();
+	}
+	{
+		knh_Glue_t *g = (knh_Glue_t*)new_Object_bcid(ctx, CLASS_Any, 0);
+		g->h.cid = cid;
+		knh_Glue_init(ctx, g, ptr, gfree);
+		return (Object*)g;
 	}
-	knh_Glue_t *g = (knh_Glue_t*)new_Object_bcid(ctx, CLASS_Any, 0);
-	g->h.cid = cid;
-	knh_Glue_init(ctx, g, ptr, gfree);
-	return (Object*)g;
-}
-
-/* ======================================================================== */
-/* [new] */
-/* @method[VIRTUAL] This! Object.new() */
-
-METHOD knh__Object_new(Ctx *ctx, knh_sfp_t *sfp)
-{
-	KNH_RETURN(ctx, sfp, sfp[0].o);
 }
 
 /* ======================================================================== */
 /* [getkey] */
 
-String *knh_Object_getkey(Ctx *ctx, Object *o)
+knh_String_t *knh_Object_getkey(Ctx *ctx, Object *o)
 {
 	knh_sfp_t lsfp;
 	lsfp.o = o;
-	lsfp.data = ((Int*)o)->n.data;
+	lsfp.data = ((knh_Int_t*)o)->n.data;
 	return ctx->share->StructTable[knh_Object_bcid(o)].fgetkey(ctx, &lsfp);
 }
 
-/* ------------------------------------------------------------------------ */
-/* @method String! Object.getKey() */
-
-METHOD knh__Object_getKey(Ctx *ctx, knh_sfp_t *sfp)
-{
-	KNH_RETURN(ctx, sfp, ctx->share->StructTable[knh_Object_bcid(sfp[0].o)].fgetkey(ctx, sfp));
-}
-
 /* ======================================================================== */
 /* [copy] */
 
@@ -157,42 +98,73 @@ Object *knh_Object_copy(Ctx *ctx, Object *b)
 /* ======================================================================== */
 /* [commons] */
 
-knh_hcode_t knh_Object_hashCode(Ctx *ctx, Object *o)
+knh_hashcode_t knh_Object_hashCode(Ctx *ctx, Object *o)
 {
-	return ctx->share->StructTable[o->h.bcid].fhashCode(ctx, o);
+	return StructTable(o->h.bcid).fhashCode(ctx, o);
 }
 
 /* ------------------------------------------------------------------------ */
 
+knh_String_t *knh_Object_key(Ctx *ctx, Object *o)
+{
+	knh_String_t *s;
+	knh_sfp_t *lsfp = ctx->esp + 1;
+	KNH_SETv(ctx, lsfp[0].o, o);
+	lsfp[0].data = knh_Object_data(o);
+	s = StructTable(o->h.bcid).fgetkey(ctx, lsfp);
+	KNH_SETv(ctx, lsfp[-1].o, s);
+	return s;
+}
+/* ------------------------------------------------------------------------ */
 
 int knh_Object_compareTo(Ctx *ctx, Object *o1, Object *o2)
 {
-	knh_class_t bcid = o1->h.bcid;
+	knh_class_t bcid1 = o1->h.bcid;
 	knh_class_t bcid2 = o2->h.bcid;
-	if(bcid == bcid2) {
-		return ctx->share->StructTable[bcid].fcompareTo(o1, o2);
+	int res;
+	if(bcid1 == bcid2) {
+		res = StructTable(bcid1).fcompareTo(ctx, o1, o2);
 	}
 	else {
 		if((o1->h.cid == CLASS_Int || o1->h.cid == CLASS_Float)
 		&& (o2->h.cid == CLASS_Int || o2->h.cid == CLASS_Float)) {
-			return (int)(knh_Number_tofloat(o1) - knh_Number_tofloat(o2));
+			res = (int)knh_Number_tofloat(o1) - (int)knh_Number_tofloat(o2);
+		}
+		else {
+			res = (int)(o1 - o2);
 		}
-		DBG2_P("Compared Incompatible Type %s - %s", CLASSN(o1->h.cid), CLASSN(o2->h.cid));
-		return (int)(o1 - o2);
 	}
+	DBG2_P("compared %s %s res=%d", CLASSNo(o1), CLASSNo(o2), res);
+	return res;
 }
 
 /* ------------------------------------------------------------------------ */
 
-int knh_Object_cmp(Object *o1, Object *o2)
+int knh_Object_compareTo2(Ctx *ctx, Object **o1, Object **o2)
 {
-	/* this funtion is prepared for the compatibility with qsort. */
-	return knh_Object_compareTo(konoha_getCurrentContext(), o1, o2);
+	Object* _o1 = *((Object**) o1);
+	Object* _o2 = *((Object**) o2);
+	knh_class_t bcid1 = _o1->h.bcid;
+	knh_class_t bcid2 = _o2->h.bcid;
+	int res;
+	if(bcid1 == bcid2) {
+		res = StructTable(bcid1).fcompareTo(ctx, _o1, _o2);
+	}
+	else {
+		if((_o1->h.cid == CLASS_Int || _o1->h.cid == CLASS_Float)
+		&& (_o2->h.cid == CLASS_Int || _o2->h.cid == CLASS_Float)) {
+			res = (int)knh_Number_tofloat(_o1) - (int)knh_Number_tofloat(_o2);
+		}
+		else {
+			res = (int)(_o1 - _o2);
+		}
+	}
+	DBG2_P("compared %s %s res=%d", CLASSNo(_o1), CLASSNo(_o2), res);
+	return res;
 }
 
 /* ------------------------------------------------------------------------ */
 
-
 knh_bool_t knh_Object_equals(Ctx *ctx, Object *o1, Object *o2)
 {
 	return (knh_Object_compareTo(ctx, o1, o2) == 0);
@@ -204,10 +176,9 @@ knh_bool_t knh_Object_equals(Ctx *ctx, Object *o1, Object *o2)
 knh_bytes_t knh_Object_tobytes(Ctx *ctx, Object *o)
 {
 	switch(o->h.bcid) {
-		case STRUCT_String : return knh_String_tobytes((String*)o);
-		case STRUCT_Bytes : return knh_Bytes_tobytes((Bytes*)o);
+		case STRUCT_String : return __tobytes((knh_String_t*)o);
+		case STRUCT_Bytes : return knh_Bytes_tobytes((knh_Bytes_t*)o);
 	}
-	TODO();
 	return STEXT("");
 }
 
