@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -38,9 +38,9 @@ extern "C" {
 /* ======================================================================== */
 /* [constructor] */
 
-KNHAPI(OutputStream*) new_OutputStream__io(Ctx *ctx, String *urn, knh_io_t fd, knh_iodrv_t *drv)
+KNHAPI(knh_OutputStream_t*) new_OutputStream__io(Ctx *ctx, knh_String_t *urn, knh_io_t fd, knh_iodrv_t *drv)
 {
-	OutputStream* o = (OutputStream*)new_Object_bcid(ctx, CLASS_OutputStream, 0);
+	knh_OutputStream_t* o = (knh_OutputStream_t*)new_Object_bcid(ctx, CLASS_OutputStream, 0);
 	KNH_SETv(ctx, DP(o)->urn, urn);
 	DP(o)->fd = fd;
 	if(fd != -1) {
@@ -53,9 +53,9 @@ KNHAPI(OutputStream*) new_OutputStream__io(Ctx *ctx, String *urn, knh_io_t fd, k
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(OutputStream*) new_OutputStream__FILE(Ctx *ctx, String *urn, FILE *fp, knh_iodrv_t *drv)
+KNHAPI(knh_OutputStream_t*) new_OutputStream__FILE(Ctx *ctx, knh_String_t *urn, FILE *fp, knh_iodrv_t *drv)
 {
-	OutputStream* o = (OutputStream*)new_Object_bcid(ctx, CLASS_OutputStream, 0);
+	knh_OutputStream_t* o = (knh_OutputStream_t*)new_Object_bcid(ctx, CLASS_OutputStream, 0);
 	KNH_SETv(ctx, DP(o)->urn, urn);
 	if(fp != NULL) {
 		DP(o)->fd = (knh_io_t)fp;
@@ -69,20 +69,20 @@ KNHAPI(OutputStream*) new_OutputStream__FILE(Ctx *ctx, String *urn, FILE *fp, kn
 /* ======================================================================== */
 /* [methods] */
 
-Object *knh_OutputStream_open(Ctx *ctx, OutputStream *o, String *urn, String *m)
+Object *knh_OutputStream_open(Ctx *ctx, knh_OutputStream_t *o, knh_String_t *urn, knh_String_t *m)
 {
-	knh_bytes_t fname = knh_String_tobytes(urn);
+	knh_bytes_t fname = __tobytes(urn);
 	knh_index_t loc = knh_bytes_index(fname, ':');
 
 	if(loc == -1 || (loc == 1 && isalpha(fname.buf[0]))) {  /* 'C:/' */
-		DP(o)->driver = konoha_getIODriver(ctx, STEXT("file"));
+		DP(o)->driver = knh_getIODriver(ctx, STEXT("file"));
 	}
 	else {
-		DP(o)->driver = konoha_getIODriver(ctx, knh_bytes_first(fname, loc));
+		DP(o)->driver = knh_getIODriver(ctx, knh_bytes_first(fname, loc));
 	}
-	char *mode = "r";
-	if(IS_NOTNULL(m)) mode = knh_String_tochar(m);
-	DP(o)->fd = DP(o)->driver->fopen(ctx, fname, mode);
+	char *mode = "w";
+	if(IS_NOTNULL(m)) mode = __tochar(m);
+	DP(o)->fd = DP(o)->driver->fopen(ctx, fname, mode, knh_Context_isStrict(ctx));
 	if(DP(o)->fd != -1) {
 		KNH_SETv(ctx, DP(o)->urn, urn);
 		KNH_SETv(ctx, DP(o)->ba, new_Bytes(ctx, 4096));
@@ -90,7 +90,7 @@ Object *knh_OutputStream_open(Ctx *ctx, OutputStream *o, String *urn, String *m)
 		DP(o)->driver->finit(ctx, (Object*)o, mode);
 	}
 	else {
-		DP(o)->driver = konoha_getDefaultIODriver();
+		DP(o)->driver = knh_getDefaultIODriver();
 	}
 	knh_OutputStream_setBOL(o,1);
 	return (Object*)o;
@@ -99,9 +99,9 @@ Object *knh_OutputStream_open(Ctx *ctx, OutputStream *o, String *urn, String *m)
 /* ======================================================================== */
 /* [methods] */
 
-void knh_OutputStream_putc(Ctx *ctx, OutputStream *o, knh_int_t ch)
+void knh_OutputStream_putc(Ctx *ctx, knh_OutputStream_t *o, knh_int_t ch)
 {
-	Bytes *ba = DP(o)->ba;
+	knh_Bytes_t *ba = DP(o)->ba;
 	KNH_ASSERT(IS_Bytes(ba));
 	knh_Bytes_putc(ctx, ba, ch);
 	if(!knh_OutputStream_isStoringBuffer(o) && ba->size > DP(o)->driver->bufsiz) {
@@ -113,9 +113,9 @@ void knh_OutputStream_putc(Ctx *ctx, OutputStream *o, knh_int_t ch)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_OutputStream_write(Ctx *ctx, OutputStream *o, knh_bytes_t buf)
+void knh_OutputStream_write(Ctx *ctx, knh_OutputStream_t *o, knh_bytes_t buf)
 {
-	Bytes *ba = DP(o)->ba;
+	knh_Bytes_t *ba = DP(o)->ba;
 	KNH_ASSERT(IS_Bytes(ba));
 	knh_Bytes_write(ctx, ba, buf);
 	if(!knh_OutputStream_isStoringBuffer(o) && ba->size > DP(o)->driver->bufsiz) {
@@ -126,21 +126,19 @@ void knh_OutputStream_write(Ctx *ctx, OutputStream *o, knh_bytes_t buf)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void OutputStream.flush() */
 
-void knh_OutputStream_flush(Ctx *ctx, OutputStream *o)
+void knh_OutputStream_flush(Ctx *ctx, knh_OutputStream_t *o)
 {
 	if(!knh_OutputStream_isStoringBuffer(o)) {
-		Bytes *ba = DP(o)->ba;
+		knh_Bytes_t *ba = DP(o)->ba;
 		DP(o)->driver->fwrite(ctx, DP(o)->fd, (char*)(ba)->buf, (ba)->size);
 		knh_Bytes_clear(ba, 0);
 	}
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void OutputStream.clear() */
 
-void knh_OutputStream_clear(Ctx *ctx, OutputStream *o)
+void knh_OutputStream_clear(Ctx *ctx, knh_OutputStream_t *o)
 {
 	if(knh_OutputStream_isStoringBuffer(o)) {
 		knh_Bytes_clear(DP(o)->ba, 0);
@@ -148,51 +146,52 @@ void knh_OutputStream_clear(Ctx *ctx, OutputStream *o)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void OutputStream.close() */
 
-void knh_OutputStream_close(Ctx *ctx, OutputStream *o)
+void knh_OutputStream_close(Ctx *ctx, knh_OutputStream_t *o)
 {
 	knh_OutputStream_flush(ctx, o);
 	f_io_close f = DP(o)->driver->fclose;
-	DP(o)->driver = konoha_getDefaultIODriver();
+	DP(o)->driver = knh_getDefaultIODriver();
 	f(ctx, DP(o)->fd);
 	DP(o)->fd = -1;
 }
 
 /* ------------------------------------------------------------------------ */
 
-int knh_OutputStream_isClosed(OutputStream *o)
+int knh_OutputStream_isClosed(knh_OutputStream_t *o)
 {
 	return (DP(o)->fd == -1);
 }
 
 /* ------------------------------------------------------------------------ */
 
-void knh_OutputStream_indent_inc(Ctx *ctx, OutputStream *o)
+void knh_OutputStream_setEncoding(Ctx *ctx, knh_OutputStream_t *o, knh_String_t *enc)
 {
-	DP(o)->indent++;
-}
-
-/* ------------------------------------------------------------------------ */
-
-void knh_OutputStream_indent_dec(Ctx *ctx, OutputStream *o)
-{
-	DP(o)->indent--;
-}
-
-/* ------------------------------------------------------------------------ */
-
-void knh_OutputStream_write_indent(Ctx *ctx, OutputStream *o)
-{
-	int i;
-	for(i = 0; i < DP(o)->indent; i++) {
-		knh_OutputStream_write(ctx, o, knh_String_tobytes(DP(o)->TAB));
+	if(IS_NULL(enc)) {
+		KNH_SETv(ctx, DP(o)->enc, TS_ENCODING);
+		KNH_SETv(ctx, DP(o)->bconv, KNH_NULL);
+	}
+	else if(knh_bytes_strcasecmp(__tobytes(enc), STEXT(KONOHA_ENCODING)) == 0) {
+		KNH_SETv(ctx, DP(o)->enc, TS_ENCODING);
+		KNH_SETv(ctx, DP(o)->bconv, KNH_NULL);
+	}
+	else {
+		knh_BytesConv_t *bout = new_BytesConv__out(ctx, __tochar(enc));
+		if(IS_NULL(bout)) {
+			KNH_SETv(ctx, DP(o)->enc, TS_ENCODING);
+			KNH_WARNING(ctx, "unsupported character encoding: %s", __tochar(enc));
+		}
+		else {
+			KNH_SETv(ctx, DP(o)->enc, enc);
+		}
+		KNH_SETv(ctx, DP(o)->bconv, bout);
 	}
 }
 
+
 /* ------------------------------------------------------------------------ */
 
-void knh_OutputStream_print_(Ctx *ctx, OutputStream *o, knh_bytes_t str, knh_bool_t isnl)
+void knh_OutputStream_print_(Ctx *ctx, knh_OutputStream_t *o, knh_bytes_t str, knh_bool_t isnl)
 {
 	if(str.len > 0) {
 		if(knh_OutputStream_isBOL(o)) {
@@ -213,20 +212,20 @@ void knh_OutputStream_print_(Ctx *ctx, OutputStream *o, knh_bytes_t str, knh_boo
 
 /* ======================================================================== */
 
-KNHAPI(OutputStream*) new_FileOutputStream(Ctx *ctx, knh_bytes_t file, char *mode)
+KNHAPI(knh_OutputStream_t*) new_FileOutputStream(Ctx *ctx, knh_bytes_t file, char *mode, int isThrowable)
 {
-	knh_iodrv_t *drv = konoha_getIODriver(ctx, STEXT("file"));
-	knh_io_t fd = drv->fopen(ctx, file, mode);
+	knh_iodrv_t *drv = knh_getIODriver(ctx, STEXT("file"));
+	knh_io_t fd = drv->fopen(ctx, file, mode, isThrowable);
 	return new_OutputStream__io(ctx, new_String(ctx, file, NULL), fd, drv);
 }
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(OutputStream*) new_BytesOutputStream(Ctx *ctx, Bytes *ba)
+KNHAPI(knh_OutputStream_t*) new_BytesOutputStream(Ctx *ctx, knh_Bytes_t *ba)
 {
-	OutputStream* o = (OutputStream*)new_Object_bcid(ctx, CLASS_OutputStream, -1);
-	knh_OutputStream_setBOL(o, 1);
+	knh_OutputStream_t* o = (knh_OutputStream_t*)new_Object_bcid(ctx, CLASS_OutputStream, -1);
 	KNH_ASSERT(IS_Bytes(ba));
+	knh_OutputStream_setBOL(o, 1);
 	KNH_SETv(ctx, DP(o)->ba, ba);
 	knh_OutputStream_setStoringBuffer(o, 1);
 	return o;
