@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -35,45 +35,27 @@
 extern "C" {
 #endif
 
-/* ======================================================================== */
-/* [macro] */
-
-#define _NSN_main           0
-#define _FILEN_unknown      0
-
-/* ======================================================================== */
-/* [constructors] */
-
-
 /* ======================================================================== */
 /* [properties] */
 
-char *
-knh_bytes_nzenvkey(knh_bytes_t t, char *buf, size_t bufsiz)
+static
+void knh_cwb_nzenvkey(Ctx *ctx, knh_cwb_t *cwb, knh_bytes_t t)
 {
 	size_t i;
 	for(i = 0; i < t.len; i++) {
-		if(i < bufsiz) {
-			buf[i] = toupper(t.buf[i]);
-		}
-		else {
-			buf[bufsiz-1] = 0;
-			return buf;
-		}
+		knh_Bytes_putc(ctx, (cwb->ba), toupper(t.buf[i]));
 	}
-	buf[i] = 0;
-	return buf;
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method Any System.getProperty(String! key) */
 
-Any* knh_System_getProperty(Ctx *ctx, System *o, knh_bytes_t key)
+Any* knh_System_getProperty(Ctx *ctx, knh_System_t *o, knh_bytes_t key)
 {
 	if(knh_bytes_startsWith(key, STEXT("env."))) {
-		char envkey[256];
-		knh_bytes_nzenvkey(knh_bytes_last(key, 4), envkey, sizeof(envkey));
-		char *v = knh_getenv(envkey);
+		knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
+		knh_cwb_nzenvkey(ctx, cwb, knh_bytes_last(key, 4));
+		char *v = knh_getenv(knh_cwb_tochar(ctx, cwb));
+		knh_cwb_close(cwb);
 		if(v == NULL) {
 			return KNH_NULL;
 		}
@@ -83,19 +65,26 @@ Any* knh_System_getProperty(Ctx *ctx, System *o, knh_bytes_t key)
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void System.setProperty(String! key, Any value) */
 
-void knh_System_setProperty(Ctx *ctx, System *o, String *key, Any *value)
+void knh_System_setProperty(Ctx *ctx, knh_System_t *o, knh_String_t *key, Any *value)
 {
-	knh_DictMap_set(ctx, DP(o)->props, key, value);
+	knh_type_t type = knh_getPropertyType(ctx, __tobytes(key));
+	if(type == TYPE_Any || knh_Object_cid(value) == CLASS_type(type)) {
+		knh_DictMap_set(ctx, DP(o)->props, key, value);
+	}
+	else {
+		knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
+		knh_printf(ctx, cwb->w, "PropertyType!!: $%s must be %T", knh_String_text(ctx, key), type);
+		knh_stack_throw(ctx, ctx->esp, knh_cwb_newException(ctx, cwb), NULL, 0);
+	}
 }
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) konoha_setSystemPropertyText(Ctx *ctx, char *key, char *value)
+KNHAPI(void) knh_setSystemPropertyText(Ctx *ctx, char *key, char *value)
 {
-	String *k = new_String__T(ctx, key);
-	String *v = new_String__T(ctx, value);
+	knh_String_t *k = T__(key);
+	knh_String_t *v = T__(value);
 	knh_DictMap_set(ctx, DP(ctx->sys)->props, k, UP(v));
 }
 
@@ -116,40 +105,8 @@ knh_bool_t konoha_initcheck(void)
  	return 1;
  }
 
-/* ======================================================================== */
-/* [urnalias] */
-
-knh_bytes_t knh_Runtime_aliasURN(Ctx *ctx, knh_bytes_t aurn)
-{
-	String *s = (String*)knh_DictMap_get__b(ctx,  DP(ctx->sys)->URNAliasDictMap, aurn);
-	if(IS_NOTNULL(s)) {
-		return knh_String_tobytes(s);
-	}
-	return aurn;
-}
-
 /* ------------------------------------------------------------------------ */
 
-void knh_Runtime_setAliasURN(Ctx *ctx, String *alias, String *urn)
-{
-	if(knh_Context_isVerbose(ctx)) {
-		String *s =(String*)knh_DictMap_get__b(ctx,  DP(ctx->sys)->URNAliasDictMap, knh_String_tobytes(urn));
-		if(IS_NOTNULL(s)) {
-			KNH_WARNING(ctx, "Overriding %s %s", knh_String_tochar(alias), knh_String_tochar(s));
-		}
-	}
-	knh_DictMap_set(ctx, DP(ctx->sys)->URNAliasDictMap, alias, UP(urn));
-}
-
-/* ------------------------------------------------------------------------ */
-
-KNHAPI(void) konoha_addAliasURN(Ctx *ctx, char *alias, char *urn)
-{
-	knh_Runtime_setAliasURN(ctx, new_String__T(ctx, alias), new_String__T(ctx, urn));
-}
-
-
-
 #ifdef __cplusplus
 }
 #endif
