@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -38,27 +38,28 @@ extern "C" {
 /* ======================================================================== */
 /* [macros] */
 
-/* ------------------------------------------------------------------------ */
-
 knh_class_t knh_pmztype_toclass(Ctx *ctx, knh_type_t t, knh_class_t this_cid)
 {
 	knh_class_t cid = CLASS_type(t);
-	KNH_ASSERT_cid(cid);
-	KNH_ASSERT_cid(this_cid);
+	DBG2_ASSERT_cid(cid);
+	DBG2_ASSERT_cid(this_cid);
 	if(cid == CLASS_This) {
 		return this_cid;
 	}
-	if(cid == CLASS_Any1) {
-		return ctx->share->ClassTable[this_cid].p1;
+	else if(cid == CLASS_T1) {
+		return ClassTable(this_cid).p1;
 	}
-	if(cid == CLASS_Any2) {
-		return ctx->share->ClassTable[this_cid].p2;
+	else if(cid == CLASS_T2) {
+		return ClassTable(this_cid).p2;
 	}
-	if(ctx->share->ClassTable[cid].bcid == CLASS_Closure) {
-		knh_type_t r0 = ctx->share->ClassTable[cid].r0;
-		knh_type_t p1 = ctx->share->ClassTable[cid].p1;
-		knh_type_t p2 = ctx->share->ClassTable[cid].p2;
-		knh_type_t p3 = ctx->share->ClassTable[cid].p3;
+	else if(cid == CLASS_Tx || cid == CLASS_Tvar) {
+		return CLASS_Any;
+	}
+	if(ClassTable(cid).bcid == CLASS_Closure) {
+		knh_type_t r0 = ClassTable(cid).r0;
+		knh_type_t p1 = ClassTable(cid).p1;
+		knh_type_t p2 = ClassTable(cid).p2;
+		knh_type_t p3 = ClassTable(cid).p3;
 		knh_type_t rr0 = knh_pmztype_totype(ctx, r0, this_cid);
 		knh_type_t pp1 = knh_pmztype_totype(ctx, p1, this_cid);
 		knh_type_t pp2 = knh_pmztype_totype(ctx, p2, this_cid);
@@ -68,13 +69,13 @@ knh_class_t knh_pmztype_toclass(Ctx *ctx, knh_type_t t, knh_class_t this_cid)
 		}
 		return cid;
 	}
-	if(knh_Class_isGenerics(cid)) { /* Iterator<This> */
-		knh_class_t p1 = ctx->share->ClassTable[cid].p1;
-		knh_class_t p2 = ctx->share->ClassTable[cid].p2;
+	if(knh_class_isGenerics(cid)) { /* Iterator<This> */
+		knh_class_t p1 = ClassTable(cid).p1;
+		knh_class_t p2 = ClassTable(cid).p2;
 		knh_class_t pp1 = knh_pmztype_toclass(ctx, p1, this_cid);
 		knh_class_t pp2 = knh_pmztype_toclass(ctx, p2, this_cid);
 		if(p1 != pp1 || p2 != pp2) {
-			return knh_class_Generics(ctx, ctx->share->ClassTable[cid].bcid, pp1, pp2);
+			return knh_class_Generics(ctx, ClassTable(cid).bcid, pp1, pp2);
 		}
 		return cid;
 	}
@@ -85,23 +86,29 @@ knh_class_t knh_pmztype_toclass(Ctx *ctx, knh_type_t t, knh_class_t this_cid)
 
 knh_type_t knh_pmztype_totype(Ctx *ctx, knh_type_t t, knh_class_t this_cid)
 {
-	knh_class_t cid = CLASS_type(t), mask = (t & KNH_FLAG_TF_NN);
-	KNH_ASSERT_cid(cid);
-	KNH_ASSERT_cid(this_cid);
+	knh_class_t cid = CLASS_type(t), mask = (t & KNH_FLAG_TF_NA);
+	DBG2_ASSERT_cid(cid);
+	DBG2_ASSERT_cid(this_cid);
 	if(cid == CLASS_This) {
 		return this_cid | mask;
 	}
-	if(cid == CLASS_Any1) {
-		return ctx->share->ClassTable[this_cid].p1 | mask;
+	if(cid == CLASS_T1) {
+		/* if aborted, setClassParam in knh_loadClassProperties(Ctx *ctx) */
+		DBG2_ASSERT(ClassTable(this_cid).p1 != CLASS_Tvoid);
+		return ClassTable(this_cid).p1 | mask;
 	}
-	if(cid == CLASS_Any2) {
-		return ctx->share->ClassTable[this_cid].p2 | mask;
+	if(cid == CLASS_T2) {
+		DBG2_ASSERT(ClassTable(this_cid).p1 != CLASS_Tvoid);
+		return ClassTable(this_cid).p2 | mask;
 	}
-	if(ctx->share->ClassTable[cid].bcid == CLASS_Closure) {
-		knh_type_t r0 = ctx->share->ClassTable[cid].r0;
-		knh_type_t p1 = ctx->share->ClassTable[cid].p1;
-		knh_type_t p2 = ctx->share->ClassTable[cid].p2;
-		knh_type_t p3 = ctx->share->ClassTable[cid].p3;
+	else if(cid == CLASS_Tx || cid == CLASS_Tvar || cid == CLASS_Any) {
+		return TYPE_Any;
+	}
+	if(ClassTable(cid).bcid == CLASS_Closure) {
+		knh_type_t r0 = ClassTable(cid).r0;
+		knh_type_t p1 = ClassTable(cid).p1;
+		knh_type_t p2 = ClassTable(cid).p2;
+		knh_type_t p3 = ClassTable(cid).p3;
 		knh_type_t rr0 = knh_pmztype_totype(ctx, r0, this_cid);
 		knh_type_t pp1 = knh_pmztype_totype(ctx, p1, this_cid);
 		knh_type_t pp2 = knh_pmztype_totype(ctx, p2, this_cid);
@@ -111,13 +118,19 @@ knh_type_t knh_pmztype_totype(Ctx *ctx, knh_type_t t, knh_class_t this_cid)
 		}
 		return cid | mask;
 	}
-	if(knh_Class_isGenerics(cid)) { /* Iterator<This> */
-		knh_class_t p1 = ctx->share->ClassTable[cid].p1;
-		knh_class_t p2 = ctx->share->ClassTable[cid].p2;
+	if(knh_class_isGenerics(cid)) { /* Iterator<This> */
+		knh_class_t p1 = ClassTable(cid).p1;
+		knh_class_t p2 = ClassTable(cid).p2;
 		knh_class_t pp1 = knh_pmztype_toclass(ctx, p1, this_cid);
-		knh_class_t pp2 = knh_pmztype_toclass(ctx, p2, this_cid);
-		if(p1 != pp1 || p2 != pp2) {
-			return knh_class_Generics(ctx, ctx->share->ClassTable[cid].bcid, pp1, pp2) | mask;
+		if(p2 != CLASS_Tvoid) {
+			knh_class_t pp2 = knh_pmztype_toclass(ctx, p2, this_cid);
+			if(p1 != pp1 || p2 != pp2) {
+				return knh_class_Generics(ctx, ClassTable(cid).bcid, pp1, pp2) | mask;
+			}
+		}else {
+			if(p1 != pp1) {
+				return knh_class_Generics(ctx, ClassTable(cid).bcid, pp1, CLASS_Tvoid) | mask;
+			}
 		}
 		return cid | mask;
 	}
@@ -129,13 +142,25 @@ knh_type_t knh_pmztype_totype(Ctx *ctx, knh_type_t t, knh_class_t this_cid)
 
 knh_class_t knh_class_Array(Ctx *ctx, knh_class_t p1)
 {
-	char buf[CLASSNAME_BUFSIZ];
+	knh_class_t bcid = CLASS_Array, cid = CLASS_unknown;
+	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
+	DBG2_ASSERT_cid(p1);
 	if(p1 == CLASS_Any) return CLASS_Array;
-	knh_snprintf(buf, sizeof(buf), "%s[]", CLASSN(p1));
-	knh_class_t cid = konoha_getcid(ctx, B(buf));
+	if(ClassTable(p1).bcid == CLASS_Int) {
+		bcid = CLASS_IArray;
+		if(p1 == CLASS_Int) return bcid;
+	}
+	if(ClassTable(p1).bcid == CLASS_Float) {
+		bcid = CLASS_FArray;
+		if(p1 == CLASS_Float) return bcid;
+	}
+	knh_write_ltype(ctx, cwb->w, p1);
+	knh_write(ctx, cwb->w, STEXT("[]"));
+	cid = knh_getcid(ctx, knh_cwb_tobytes(cwb));
 	if(cid == CLASS_unknown) {
-		//DBG2_P("Generating %s", buf);
-		return konoha_addGenericsClass(ctx, CLASS_newid, new_String(ctx, B(buf), NULL), CLASS_Array, p1, CLASS_Nue);
+		cid = knh_addGenericsClass(ctx, CLASS_newid, knh_cwb_newString(ctx, cwb), bcid, p1, CLASS_Tvoid);
+	} else {
+		knh_cwb_close(cwb);
 	}
 	return cid;
 }
@@ -144,13 +169,16 @@ knh_class_t knh_class_Array(Ctx *ctx, knh_class_t p1)
 
 knh_class_t knh_class_Iterator(Ctx *ctx, knh_class_t p1)
 {
-	char buf[CLASSNAME_BUFSIZ];
+	knh_class_t cid = CLASS_unknown;
+	knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
 	if(p1 == CLASS_Any) return CLASS_Iterator;
-	knh_snprintf(buf, sizeof(buf), "%s..", CLASSN(p1));
-	knh_class_t cid = konoha_getcid(ctx, B(buf));
+	knh_write_ltype(ctx, cwb->w, p1);
+	knh_write(ctx, cwb->w, STEXT(".."));
+	/* knh_class_t*/ cid = knh_getcid(ctx, knh_cwb_tobytes(cwb));
 	if(cid == CLASS_unknown) {
-		//DBG2_P("*** Generating %s ***", buf);
-		return konoha_addGenericsClass(ctx, CLASS_newid, new_String(ctx, B(buf), NULL), CLASS_Iterator, p1, CLASS_Nue);
+		cid = knh_addGenericsClass(ctx, CLASS_newid, knh_cwb_newString(ctx, cwb), CLASS_Iterator, p1, CLASS_Tvoid);
+	} else {
+		knh_cwb_close(cwb);
 	}
 	return cid;
 }
@@ -159,23 +187,32 @@ knh_class_t knh_class_Iterator(Ctx *ctx, knh_class_t p1)
 
 knh_class_t knh_class_Generics(Ctx *ctx, knh_class_t bcid, knh_class_t p1, knh_class_t p2)
 {
-	DBG2_ASSERT(bcid != CLASS_Closure);
 	if(bcid == CLASS_Iterator) {
 		return knh_class_Iterator(ctx, p1);
 	}
-	else if(bcid == CLASS_Array){
+	else if(bcid == CLASS_Array || bcid == CLASS_IArray || bcid == CLASS_FArray) {
 		return knh_class_Array(ctx, p1);
 	}
-	if(p1 == CLASS_Any && (p2 == CLASS_Any || p2 == CLASS_Nue)) {
+	if(p1 == CLASS_Any && (p2 == CLASS_Any || p2 == CLASS_Tvoid)) {
 		return bcid;
 	}
 	else {
-		char buf[CLASSNAME_BUFSIZ];
-		knh_snprintf(buf, sizeof(buf), "%s<%s,%s>", CLASSN(bcid), CLASSN(p1), CLASSN(p2));
-		knh_class_t cid = konoha_getcid(ctx, B(buf));
+		knh_class_t cid;
+		knh_cwb_t cwbbuf, *cwb = knh_cwb_open(ctx, &cwbbuf);
+		knh_write_ltype(ctx, cwb->w, bcid);
+		knh_putc(ctx, cwb->w, '<');
+		knh_write_ltype(ctx, cwb->w, p1);
+		if(p2 != CLASS_Tvoid) {
+			knh_putc(ctx, cwb->w, ',');
+			knh_write_ltype(ctx, cwb->w, p2);
+
+		}
+		knh_putc(ctx, cwb->w, '>');
+		/* knh_class_t*/ cid = knh_getcid(ctx, knh_cwb_tobytes(cwb));
 		if(cid == CLASS_unknown) {
-			//DBG2_P("Generating %s", buf);
-			return konoha_addGenericsClass(ctx, CLASS_newid, new_String(ctx, B(buf), NULL), bcid, p1, p2);
+			cid = knh_addGenericsClass(ctx, CLASS_newid, knh_cwb_newString(ctx, cwb), bcid, p1, p2);
+		} else {
+			knh_cwb_close(cwb);
 		}
 		return cid;
 	}
@@ -183,40 +220,6 @@ knh_class_t knh_class_Generics(Ctx *ctx, knh_class_t bcid, knh_class_t p1, knh_c
 
 /* ------------------------------------------------------------------------ */
 
-char* knh_format_type(Ctx *ctx, char *buf, size_t bufsiz, knh_type_t type)
-{
-	if(type == TYPE_void) {
-		knh_snprintf(buf, bufsiz, "void");
-		return buf;
-	}
-	if(type == TYPE_void) {
-		knh_snprintf(buf, bufsiz, "var");
-		return buf;
-	}
-	knh_class_t cid = CLASS_type(type);
-	if(!(cid < ctx->share->ClassTableSize)) {
-		cid = CLASS_Any;
-		type = cid;
-	}
-	KNH_ASSERT_cid(cid);
-	char *cname = knh_ClassTable_CLASSN(ctx, cid);
-	if(IS_ubxtype(type)) {
-		knh_snprintf(buf, bufsiz, "%s", cname);
-		buf[0] = tolower(buf[0]);
-		return buf;
-	}
-	if(IS_NNTYPE(type)) {
-		knh_snprintf(buf, bufsiz, "%s", cname);
-	}
-	else {
-		knh_snprintf(buf, bufsiz, "%s?", cname);
-	}
-	return buf;
-}
-
-/* ======================================================================== */
-/* [movabletext] */
-
 char *TYPEQ(knh_type_t type)
 {
 	if(IS_NNTYPE(type)) {
@@ -227,8 +230,6 @@ char *TYPEQ(knh_type_t type)
 
 /* ------------------------------------------------------------------------ */
 
-#define _TYPEN(type)  knh_TYPEN(ctx,type)
-
 char *knh_TYPEN(Ctx *ctx, knh_type_t type)
 {
 	if(type == TYPE_void) return "void";
