@@ -41,18 +41,18 @@ extern "C" {
 /* ------------------------------------------------------------------------ */
 /* @ method void Script.changeChannel(Int! ch) */
 
-METHOD knh__Script_changeChannel(Ctx *ctx, knh_sfp_t *sfp)
+METHOD Script_changeChannel(Ctx *ctx, knh_sfp_t *sfp)
 {
 	fprintf(stdout, "switching channel to %d..\n", (int)sfp[1].ivalue);
-	konoha_throwSecurityException();
+	//konoha_throwSecurityException();
 }
 
 /* ------------------------------------------------------------------------ */
 /* @ method void Script.hook(String! funcname) */
 
-METHOD knh__Script_hook(Ctx *ctx, knh_sfp_t *sfp)
+METHOD Script_hook(Ctx *ctx, knh_sfp_t *sfp)
 {
-	char *funcname = knh_String_tochar(sfp[1].s);
+	char *funcname = __tochar(sfp[1].s);
 	fprintf(stdout, "hooking security point at %s\n", funcname);
 }
 
