@@ -1,7 +1,7 @@
 /****************************************************************************
  * KONOHA COPYRIGHT, LICENSE NOTICE, AND DISCRIMER
  *
- * Copyright (c) 2005-2008, Kimio Kuramitsu <kimio at ynu.ac.jp>
+ * Copyright (c) 2005-2009, Kimio Kuramitsu <kimio at ynu.ac.jp>
  *           (c) 2008-      Konoha Software Foundation
  * All rights reserved.
  *
@@ -28,7 +28,9 @@
 /* ************************************************************************ */
 
 #include"commons.h"
+#ifndef KONOHA_ON_LKM
 #include<time.h>
+#endif
 
 /* ************************************************************************ */
 
@@ -39,17 +41,8 @@ extern "C" {
 /* ======================================================================== */
 /* [verbose] */
 
-#define _KNH_FLAG_PF_STDERR      KNH_FLAG_T0
-#define _KNH_FLAG_PF_EOL         KNH_FLAG_T1
-#define _KNH_FLAG_PF_TIME        KNH_FLAG_T2
-#define _KNH_FLAG_PF_FUNC        KNH_FLAG_T3
-#define _KNH_FLAG_PF_NAME        KNH_FLAG_T4
-#define _KNH_FLAG_PF_BOL         KNH_FLAG_T7
-
-/* ------------------------------------------------------------------------ */
-
 static
-OutputStream *knh_stack_pstream(Ctx *ctx, knh_flag_t flag)
+knh_OutputStream_t *knh_stack_pstream(Ctx *ctx, knh_flag_t flag)
 {
 	return KNH_STDOUT;
 }
@@ -57,7 +50,7 @@ OutputStream *knh_stack_pstream(Ctx *ctx, knh_flag_t flag)
 /* ------------------------------------------------------------------------ */
 
 static
-void knh_stack_beginPRINT(Ctx *ctx, knh_sfp_t *sfp, knh_flag_t flag, OutputStream *w)
+void knh_stack_beginPRINT(Ctx *ctx, knh_flag_t flag, knh_OutputStream_t *w)
 {
 	if(KNH_FLAG_IS(flag, KNH_FLAG_PF_BOL)) {
 		if(KNH_FLAG_IS(flag, KNH_FLAG_PF_TIME)) {
@@ -69,26 +62,23 @@ void knh_stack_beginPRINT(Ctx *ctx, knh_sfp_t *sfp, knh_flag_t flag, OutputStrea
 	//		knh_snprintf(buf, sizeof(buf), "[%d/%d/%d %d/%d/%d] ", now->tm_year, now->tm_mon, now->tm_mday, now->tm_hour, now->tm_min, now->tm_sec);
 	//		knh_write__s(ctx, w, buf);
 		}
-		knh_putc(ctx, w, '[');
 	}
 }
 
 /* ------------------------------------------------------------------------ */
 
 static
-void knh_stack_endPRINT(Ctx *ctx, knh_sfp_t *sfp, knh_flag_t flag, OutputStream *w)
+void knh_stack_endPRINT(Ctx *ctx, knh_flag_t flag, knh_OutputStream_t *w)
 {
-	if(KNH_FLAG_IS(flag, KNH_FLAG_PF_BOL)) {
-		knh_putc(ctx, w, ']');
-		knh_putc(ctx, w, ' ');
-	}
-	else if(KNH_FLAG_IS(flag, KNH_FLAG_PF_EOL)) {
+	if(KNH_FLAG_IS(flag, KNH_FLAG_PF_EOL)) {
 		knh_write_EOL(ctx, w);
-		knh_flush(ctx, w);
 	}
 	else if(KNH_FLAG_IS(flag, KNH_FLAG_PF_NAME)) {
 		knh_putc(ctx, w, '=');
 	}
+	else if(KNH_FLAG_IS(flag, KNH_FLAG_PF_BOL)) {
+		knh_putc(ctx, w, ' ');
+	}
 	else {
 		knh_putc(ctx, w, ',');
 		knh_putc(ctx, w, ' ');
@@ -97,55 +87,49 @@ void knh_stack_endPRINT(Ctx *ctx, knh_sfp_t *sfp, knh_flag_t flag, OutputStream
 
 /* ------------------------------------------------------------------------ */
 
-void knh_stack_pmsg(Ctx *ctx, knh_sfp_t *sfp, knh_flag_t flag, String *s)
+void knh_stack_pmsg(Ctx *ctx, knh_flag_t flag, knh_String_t *s)
 {
-	OutputStream *w = knh_stack_pstream(ctx, flag);
-	knh_stack_beginPRINT(ctx, sfp, flag, w);
-	if(IS_bString(s)) {
-		knh_print(ctx, w, knh_String_tobytes(s));
+	knh_OutputStream_t *w = knh_stack_pstream(ctx, flag);
+	knh_stack_beginPRINT(ctx, flag, w);
+	DBG2_ASSERT(IS_bString(s));
+	if((s)->size > 0) {
+		knh_print(ctx, w, __tobytes(s));
 	}
-	knh_stack_endPRINT(ctx, sfp, flag, w);
+//	else {
+//		flag = (flag & ~(KNH_FLAG_PF_BOL));
+//	}
+	knh_stack_endPRINT(ctx, flag, w);
 }
 
 /* ------------------------------------------------------------------------ */
 
-void knh_stack_p(Ctx *ctx, knh_sfp_t *sfp, knh_flag_t flag, knh_methodn_t mn, int sfpidx)
+void knh_stack_p(Ctx *ctx, knh_sfp_t *sfp, knh_flag_t flag, knh_methodn_t mn)
 {
-	OutputStream *w = knh_stack_pstream(ctx, flag);
-	knh_stack_beginPRINT(ctx, sfp, flag, w);
-	knh_sfp_t *esp = ctx->esp;
-	KNH_SETv(ctx, esp[1].o, sfp[sfpidx].o);
-	esp[1].data = sfp[sfpidx].data;
-	knh_esp1_format(ctx, mn, w, KNH_NULL);
-	knh_stack_endPRINT(ctx, sfp, flag, w);
+	knh_OutputStream_t *w = knh_stack_pstream(ctx, flag);
+	knh_stack_beginPRINT(ctx, flag, w);
+	knh_Method_t *mtd = knh_lookupFormatter(ctx, knh_Object_cid(sfp[1].o), mn);
+	KNH_SETv(ctx, sfp[2].o, w);
+	KNH_SETv(ctx, sfp[3].o, KNH_NULL);
+	KNH_SCALL(ctx, sfp, 0, mtd, /*args*/2);
+	knh_stack_endPRINT(ctx, flag, w);
 }
 
 /* ------------------------------------------------------------------------ */
-
-static int tglobalVerbose = 5;
-
-/* ------------------------------------------------------------------------ */
-
-int knh_verbose(void)
-{
-	return tglobalVerbose;
-}
-
-/* ======================================================================== */
 /* [file] */
 
-KNHAPI(char*) KNH_SAFEFILE(char *file)
+KNHAPI(char*) knh_safefile(char *file)
 {
-	if(file == NULL) return "(unknown)";
-	knh_bytes_t t = B(file);
-	int loc = knh_bytes_rindex(t, '/');
-	if(loc == -1) {
-		return file;
+	if(file != NULL) {
+		knh_bytes_t t = B(file);
+		int loc = knh_bytes_rindex(t, '/');
+		if(loc == -1) {
+			return file;
+		}
+		return (char*)t.buf + loc + 1;
 	}
-	return (char*)t.buf + loc + 1;
+	return "(unknown)";
 }
 
-
 /* ------------------------------------------------------------------------ */
 
 #ifdef __cplusplus
